import 'package:easy_alert/easy_alert.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum ToastPosition {
  bottom,
  center,
  top,
}

enum ToastDuration {
  short,
  long,
}

class Toast {
  final String message;
  final ToastPosition position;
  final ToastDuration duration;

  Toast({this.position, this.message, this.duration});
}

class ToastView extends StatelessWidget {
  final String text;
  final TextDirection textDirection;

  ToastView({this.text, this.textDirection});

  @override
  Widget build(BuildContext context) {
    return new ClipRRect(
      borderRadius: new BorderRadius.circular(5.0),
      child: new Container(
        color: Colors.black54,
        child: new Padding(
          padding: new EdgeInsets.fromLTRB(20, 8, 20, 8.0),
          child: new Text(
            text,
            textDirection: textDirection,
            textAlign: TextAlign.center,
            style: new TextStyle(fontSize: 13.0, color: Colors.white),
          ),
        ),
      ),
    );
  }
}

class ToastControlView extends StatefulWidget {
  final AlertConfig config;
  final TextDirection textDirection;
  final ToastController controller;

  ToastControlView({this.config, this.textDirection, this.controller});

  @override
  _ToastControlViewState createState() => _ToastControlViewState();
}

class ToastController implements ToastManager {
  ToastManager target;

  @override
  void showToast(String message,
      {ToastPosition position, ToastDuration duration = ToastDuration.short}) {
    target.showToast(message, position: position, duration: duration);
  }
}

class _ToastControlViewState extends State<ToastControlView>
    with SingleTickerProviderStateMixin
    implements ToastManager {
  List<Toast> _queue = [];
  Toast _current;

  AnimationController _controller;
  Animation<double> _animation;

  @override
  void showToast(
    String message, {
    ToastPosition position,
    ToastDuration duration: ToastDuration.short,
  }) {
    Toast toast =
        new Toast(message: message, position: position, duration: duration);
    if (_current != null) {
      _queue.add(toast);
    } else {
      _show(toast);
    }
  }

  void _show(Toast toast) {
    _current = toast;
    _controller.animateTo(1.0,
        curve: Curves.ease, duration: new Duration(milliseconds: 300));
    new Future.delayed(new Duration(
            milliseconds:
                _current.duration == ToastDuration.long ? 3000 : 1000))
        .whenComplete(_hide);
    setState(() {});
  }

  void _hide() {
    _controller
        .animateTo(0.0,
            curve: Curves.ease, duration: new Duration(milliseconds: 300))
        .whenComplete(() {
      if (_queue.length > 0) {
        _show(_queue.removeAt(0));
        return;
      } else {
        _current = null;
        setState(() {});
      }
    });
  }

  @override
  void initState() {
    widget.controller.target = this;
    _controller = new AnimationController(vsync: this);
    _controller.duration = new Duration(milliseconds: 300);
    _animation = new Tween(begin: 0.0, end: 1.0).animate(_controller);
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  AlignmentGeometry _getAlignment() {
    switch (_current.position) {
      case ToastPosition.center:
        return Alignment.center;
      case ToastPosition.bottom:
        return Alignment.bottomCenter;
      case ToastPosition.top:
        return Alignment.topCenter;
    }
    return Alignment.center;
  }

  @override
  Widget build(BuildContext context) {
    return _current == null
        ? new Container()
        : new Padding(
            padding: widget.config.toastPadding,
            child: new Align(
              child: new AnimatedBuilder(
                  animation: _animation,
                  builder: (BuildContext context, Widget w) {
                    return new Opacity(
                      opacity: _animation.value,
                      child: new IgnorePointer(
                        child: new ToastView(
                          text: _current.message,
                          textDirection: widget.textDirection,
                        ),
                      ),
                    );
                  }),
              alignment: _getAlignment(),
            ));
  }
}

abstract class ToastManager {
  void showToast(
    String message, {
    ToastPosition position,
    ToastDuration duration: ToastDuration.short,
  });
}
